'''
This bottle-sqlalchemy plugin integrates SQLAlchemy with your Bottle
application. It connects to a database at the beginning of a request,
passes the database handle to the route callback and closes the connection
afterwards.

The plugin inject an argument to all route callbacks that require a `db`
keyword.

Usage Example::

    import bottle
    from bottle import HTTPError
    from bottle.ext import sqlalchemy
    from sqlalchemy import create_engine, Column, Integer, Sequence, String
    from sqlalchemy.ext.declarative import declarative_base

    Base = declarative_base()
    engine = create_engine('sqlite:///:memory:', echo=True)

    app = bottle.Bottle()
    plugin = sqlalchemy.Plugin(engine, Base.metadata, create=True)
    app.install(plugin)

    class Entity(Base):
        __tablename__ = 'entity'
        id = Column(Integer, Sequence('id_seq'), primary_key=True)
        name = Column(String(50))

        def __init__(self, name):
            self.name = name

        def __repr__(self):
            return "<Entity('%d', '%s')>" % (self.id, self.name)


    @app.get('/:name')
    def show(name, db):
        entity = db.query(Entity).filter_by(name=name).first()
        if entity:
            return {'id': entity.id, 'name': entity.name}
        return HTTPError(404, 'Entity not found.')

    @app.put('/:name')
    def put_name(name, db):
        entity = Entity(name)
        db.add(entity)


It is up to you create engine and metadata, because SQLAlchemy has
a lot of options to do it. The plugin just handles the SQLAlchemy
session.

Copyright (c) 2011-2012, Iuri de Silvio
License: MIT (see LICENSE for details)
'''

import inspect

import bottle
from sqlalchemy.exc import SQLAlchemyError
from sqlalchemy.orm import sessionmaker
from sqlalchemy.orm.scoping import ScopedSession

# PluginError is defined to bottle >= 0.10
if not hasattr(bottle, 'PluginError'):

    class PluginError(bottle.BottleException):
        pass

    bottle.PluginError = PluginError


class SQLAlchemyPlugin(object):

    name = 'sqlalchemy'
    api = 2

    def __init__(self, engine, metadata=None, keyword='db', commit=True,
                 create=False, use_kwargs=False, create_session=None):
        '''
        :param engine: SQLAlchemy engine created with `create_engine` function
        :param metadata: SQLAlchemy metadata. It is required only if `create=True`
        :param keyword: Keyword used to inject session database in a route
        :param create: If it is true, execute `metadata.create_all(engine)`
               when plugin is applied
        :param commit: If it is true, commit changes after route is executed.
        :param use_kwargs: plugin inject session database even if it is not
               explicitly defined, using **kwargs argument if defined.
        :param create_session: SQLAlchemy session maker created with the 
                'sessionmaker' function. Will create its own if undefined.
        '''
        self.engine = engine
        if create_session is None:
            create_session = sessionmaker()
        self.create_session = create_session
        self.metadata = metadata
        self.keyword = keyword
        self.create = create
        self.commit = commit
        self.use_kwargs = use_kwargs

    def setup(self, app):
        ''' Make sure that other installed plugins don't affect the same
            keyword argument and check if metadata is available.'''
        for other in app.plugins:
            if not isinstance(other, SQLAlchemyPlugin):
                continue
            if other.keyword == self.keyword:
                raise bottle.PluginError("Found another SQLAlchemy plugin with "\
                                  "conflicting settings (non-unique keyword).")
            elif other.name == self.name:
                self.name += '_%s' % self.keyword
        if self.create and not self.metadata:
            raise bottle.PluginError('Define metadata value to create database.')

    def apply(self, callback, route):
        # hack to support bottle v0.9.x
        if bottle.__version__.startswith('0.9'):
            config = route['config']
            _callback = route['callback']
        else:
            config = route.config
            _callback = route.callback

        if "sqlalchemy" in config:  # support for configuration before `ConfigDict` namespaces
            g = lambda key, default: config.get('sqlalchemy', {}).get(key, default)
        else:
            g = lambda key, default: config.get('sqlalchemy.' + key, default)

        keyword = g('keyword', self.keyword)
        create = g('create', self.create)
        commit = g('commit', self.commit)
        use_kwargs = g('use_kwargs', self.use_kwargs)

        try:
            # check if inspect.signature exists
            inspect.signature
        except AttributeError:
            argspec = inspect.getargspec(_callback)
            parameters = argspec.args
            accept_kwargs = argspec.keywords
        else:
            parameters = inspect.signature(_callback).parameters
            accept_kwargs = any(p.kind == inspect.Parameter.VAR_KEYWORD
                                for p in parameters.values())

        if not ((use_kwargs and accept_kwargs) or keyword in parameters):
            return callback

        if create:
            self.metadata.create_all(self.engine)

        def wrapper(*args, **kwargs):
            kwargs[keyword] = session = self.create_session(bind=self.engine)
            try:
                rv = callback(*args, **kwargs)
                if commit:
                    session.commit()
            except (SQLAlchemyError, bottle.HTTPError):
                session.rollback()
                raise
            except bottle.HTTPResponse:
                if commit:
                    session.commit()
                raise
            finally:
                if isinstance(self.create_session, ScopedSession):
                    self.create_session.remove()
                else:
                    session.close()
            return rv

        return wrapper


Plugin = SQLAlchemyPlugin



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# This script deletes all tests/examples/result_* directories.
# ---------------------------------------------------------------------

from __future__ import print_function

import os
import sys
from os.path import abspath, basename, dirname, join

PY2 = sys.version_info[0] == 2
PY3 = sys.version_info[0] == 3


def main():
    exdir = join(dirname(dirname(abspath(__file__))), 'tests', 'examples')
    print(exdir)
    for path, dirs, files in os.walk(exdir):
        base = basename(path)
        if not base.startswith('result_'):
            continue
        for name in files:
            filename = os.path.join(path, name)
            print("Deleting %s" % filename)
            os.remove(filename)
        print("Deleting %s" % path)
        os.rmdir(path)


if __name__ == '__main__':
    sys.exit(main())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# This script extract options for scalafmt from the HTML documentation on
# https://scalameta.org/scalafmt.
# scalafmt use HOCON syntax for the configuration which supports nesting
# options. We only extract the simple options that can be expressed as
# key-value pairs.
# ---------------------------------------------------------------------

from __future__ import print_function

import os
import re
import sys
from os.path import abspath, basename, dirname, join


re_option = re.compile(r'([a-z]\w\w+(\.\w\w\w+(\.\w\w\w+)?)?)\s*=\s*[0-9a-zA-Z]+')


def main():
    if len(sys.argv) != 2:
        print('Usage: %s <path to Scalafmt.html>')
        sys.exit(1)
    htmlfile = sys.argv[1]
    with open(htmlfile) as fp:
        lines = fp.readlines()
    options = []
    ready = False
    for line in lines:
        # Begin in the configuration section
        if '#Configuration' in line:
            ready = True
        # Stop at gotchas
        if '#Gotchas' in line:
            ready = False
        if not ready:
            continue

        m = re_option.search(line)
        if m:
            options.append(m.group(0))
    print('\n'.join(sorted(options)))


if __name__ == '__main__':
    sys.exit(main())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# The code formatter 'indent' exists in several variants with similar
# options.
# The script finds valid command line options given the ASCII manpages
# which must reside in a subdirectory called 'indenters'.
# To get the manpages in ASCII we use: man indent | col -bx
# ---------------------------------------------------------------------

from __future__ import print_function

import os
import re
import sys
from os.path import abspath, dirname, join
from pprint import pprint

PY2 = sys.version_info[0] == 2
PY3 = sys.version_info[0] == 3


def get_option_pairs(filename):
    """Returns a list of option pairs in the file.
    Each option pair falls into three categories:
    1) (-opt, None) : This is a simple option
    2) (-opt, -negative-opt): An option and its opposite option.
    3) (-opt, --longopt): An option with its longopt equivalent (only used by GNU indent).
    """
    re_dash = re.compile(r'^(\s+)(-[a-z0-9]+)(?:, (-[\w-]+))?')
    pairs = []
    with open(filename) as fp:
        for line in fp.readlines():
            line = line.rstrip()
            m = re_dash.match(line)
            if not m:
                continue
            spc, opt1, opt2 = m.groups()
            if len(spc) >= 10:
                # option is part of the description block
                continue
            if opt1 == '-v':
                # -v sort of is the last option for all indent variants.
                break
            pairs.append((opt1, opt2))
    return pairs


def classify_options(optset):
    handled = set()
    pairs = []
    for pair in ['br bl', 'brs bls', 'brf blf', 'bbb nbbb', 'bfde nbfde', 'lps nlps']:
        a, b = pair.split()
        pairs.append((a, b))
        handled.add(a)
        handled.add(b)
    # Ignore these
    handled.update(set('pmt troff ta v nv'.split()))
    # Ignore the predefined styles
    handled.update(set('orig linux kr gnu'.split()))

    optnames = set([opt[1:] for opt in optset])
    options = []
    for optname in optnames:
        if optname in handled:
            continue
        numeric = optname.endswith('n')
        isopposite = optname.startswith('n')
        theopposite = optname[1:] if isopposite else 'n' + optname
        has_opposite = theopposite in optnames
        if has_opposite:
            if not isopposite:
                pairs.append((optname, theopposite))
                handled.add(optname)
                handled.add(theopposite)
        elif numeric:
            optname = optname[:-1]
            options.append((optname, 'int', tuple()))
            handled.add(optname)
        else:
            options.append((optname, 'enum', (optname, )))
            handled.add(optname)
    for a, b in pairs:
        options.append((a, 'enum', (a, b)))

    return options


def parse_options():
    srcdir = join(dirname(abspath(__file__)), 'indenters')
    manpages = reversed(os.listdir(srcdir))
    opts = set()
    for filename in manpages:
        pairs = get_option_pairs(join(srcdir, filename))
        for opt1, opt2 in pairs:
            opts.add(opt1)
            if opt2 is not None and not opt2.startswith('--'):
                opts.add(opt2)
        copts = classify_options(opts)
    pprint(sorted(copts))

# This is the result for the man pages of indent for OS X, OpenBSD, FreeBSD and GNU indent:
# [('bacc', 'enum', ('bacc', 'nbacc')),
#  ('bad', 'enum', ('bad', 'nbad')),
#  ('bap', 'enum', ('bap', 'nbap')),
#  ('bbb', 'enum', ('bbb', 'nbbb')),
#  ('bbo', 'enum', ('bbo', 'nbbo')),
#  ('bc', 'enum', ('bc', 'nbc')),
#  ('bfda', 'enum', ('bfda', 'nbfda')),
#  ('bfde', 'enum', ('bfde', 'nbfde')),
#  ('bli', 'int', ()),
#  ('br', 'enum', ('br', 'bl')),
#  ('brf', 'enum', ('brf', 'blf')),
#  ('brs', 'enum', ('brs', 'bls')),
#  ('bs', 'enum', ('bs',)),
#  ('c', 'int', ()),
#  ('cbi', 'int', ()),
#  ('cd', 'int', ()),
#  ('cdb', 'enum', ('cdb', 'ncdb')),
#  ('cdw', 'enum', ('cdw', 'ncdw')),
#  ('ce', 'enum', ('ce', 'nce')),
#  ('ci', 'int', ()),
#  ('cli', 'int', ()),
#  ('cp', 'int', ()),
#  ('cs', 'enum', ('cs', 'ncs')),
#  ('d', 'int', ()),
#  ('di', 'int', ()),
#  ('dj', 'enum', ('dj', 'ndj')),
#  ('djn', 'enum', ('djn', 'ndjn')),
#  ('eei', 'enum', ('eei', 'neei')),
#  ('ei', 'enum', ('ei', 'nei')),
#  ('fbs', 'enum', ('fbs', 'nfbs')),
#  ('fc1', 'enum', ('fc1', 'nfc1')),
#  ('fca', 'enum', ('fca', 'nfca')),
#  ('fcb', 'enum', ('fcb', 'nfcb')),
#  ('hnl', 'enum', ('hnl', 'nhnl')),
#  ('i', 'int', ()),
#  ('il', 'int', ()),
#  ('ip', 'int', ()),
#  ('l', 'int', ()),
#  ('lc', 'int', ()),
#  ('ldi', 'int', ()),
#  ('lp', 'enum', ('lp', 'nlp')),
#  ('lps', 'enum', ('lps', 'nlps')),
#  ('npro', 'enum', ('npro',)),
#  ('pcs', 'enum', ('pcs', 'npcs')),
#  ('pi', 'int', ()),
#  ('ppi', 'int', ()),
#  ('prs', 'enum', ('prs', 'nprs')),
#  ('psl', 'enum', ('psl', 'npsl')),
#  ('saf', 'enum', ('saf', 'nsaf')),
#  ('sai', 'enum', ('sai', 'nsai')),
#  ('saw', 'enum', ('saw', 'nsaw')),
#  ('sbi', 'int', ()),
#  ('sc', 'enum', ('sc', 'nsc')),
#  ('sob', 'enum', ('sob', 'nsob')),
#  ('ss', 'enum', ('ss', 'nss')),
#  ('st', 'enum', ('st',)),
#  ('ts', 'int', ()),
#  ('ut', 'enum', ('ut', 'nut'))]


def main():
    parse_options()


if __name__ == '__main__':
    sys.exit(main())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# This script uses a repurposed version of docs/tools/dump_format_style.py
# from http://llvm.org/git/clang.git
# to generate a version history of clang-format options for all commits.
#
# The file include/clang/Format/Format.h contains the ground truth about
# the options for each clang-format version.
#
# This script takes a clang repository as only argument, e.g. llvm/tools/clang
# and needs 'git' in the PATH.
#
# The output is a diff-like representation that can be copied into whatstyle.py
# under CLANG_FORMAT_EVOLUTION to support new clang-formats options.
# The output looks like this:
# ---------------------------------------------------------------------
# # Clang bac016bd3f67ca2f4db1ddc619e611759352b84d
# + BasedOnStyle string
#         LLVM
#         Google
# + AccessModifierOffset int
# + ColumnLimit unsigned
# + MaxEmptyLinesToKeep unsigned
# + PointerAndReferenceBindToType bool
# + SplitTemplateClosingGreater bool
# # Clang 15757316d67cb7a854d53a0402d67ad58347600a
# + IndentCaseLabels bool
# # Clang 3.3
# + SpacesBeforeTrailingComments unsigned
# ...
# ---------------------------------------------------------------------

from __future__ import print_function

import sys

if (((sys.version_info[0] == 2) and (sys.version_info[1] < 7)) or (
    (sys.version_info[0] == 3) and (sys.version_info[1] < 2))):
    sys.stderr.write('Error: Python 2.7 or when running on Python 3 at least Python 3.2'
                     ' is required to run whatstyle\n')
    sys.exit(1)

import argparse
import codecs
import re
import subprocess
from collections import OrderedDict, namedtuple
from pprint import pprint

import dumpformatoptions

FIRST_SUPPORTED_VERSION = '3.5'
# Uncomment the next line to support ancient clang versions.
# FIRST_SUPPORTED_VERSION = None

PY2 = sys.version_info[0] == 2
PY3 = sys.version_info[0] == 3

if PY2:
    text_type = unicode
    binary_type = str
else:
    text_type = str
    binary_type = bytes


def unistr(text):
    if not isinstance(text, text_type):
        text = text.decode('utf-8')
    return text


def bytestr(text):
    if not isinstance(text, binary_type):
        text = text.encode('utf-8')
    return text


def outline(s, fp=None):
    if fp is None:
        fp = sys.stdout
    fp.write(unistr(s + '\n'))


reportmessage = outline


def reporterror(s, fp=None):
    if fp is None:
        fp = sys.stderr
    reportmessage(s, fp=fp)


reportwarning = reporterror

ExeResult = namedtuple('ExeResult', ['returncode', 'stdout', 'stderr'])


def run_executable(executable, cmdargs, stdindata=None):
    proc = subprocess.Popen([executable] + cmdargs,
                            stdin=subprocess.PIPE,
                            stdout=subprocess.PIPE,
                            stderr=subprocess.PIPE)
    stdout, stderr = proc.communicate(input=stdindata)
    return ExeResult(proc.returncode, stdout, stderr)


def gitcmd(cmdargs):
    return run_executable('git', cmdargs)


def gitcat(cwd, commit, filename):
    """Returns the file content for a commit.
    """
    ret, stdout, stderr = gitcmd(['-C', cwd, 'show', '%s:%s' % (commit, unistr(filename))])
    return stdout


def git_commits(cwd, *relpaths):
    """Return a list of commit hashes for relpath sorted from newest to oldest.
    """
    args = ['-C', cwd, 'log', '--pretty=oneline']
    args.extend(relpaths)
    ret, stdout, stderr = gitcmd(args)
    lines = unistr(stdout).splitlines()
    return list([l.split()[0] for l in lines])


def git_format_commits(cwd):
    """Yields a triple of
    (commithash, content of Format.h, content of docs/conf.py)
    for each commit of Format.h.
    """
    relpaths = 'include/clang/Format/Format.h include/clang/Tooling/Inclusions/IncludeStyle.h'.split()
    for commit in reversed(git_commits(cwd, *relpaths)):
        format_h = unistr(gitcat(cwd, commit, relpaths[0]))
        includestyle_h = unistr(gitcat(cwd, commit, relpaths[1]))
        conf_py = unistr(gitcat(cwd, commit, 'docs/conf.py'))
        yield commit, format_h, includestyle_h, conf_py


def parse_options(format_h_lines, includestyle_h):
    """Parses the options from the lines of Format.h
    by using a modified version of clangs dumpformatoption.py.

    Returns the options and a list of unknown option types.
    """
    unknown_optiontypes = []

    def isknownoptiontype(optiontype):
        is_known_type = optiontype in [
            'bool', 'unsigned', 'int', 'std::string', 'std::vector<std::string>',
            'std::vector<IncludeCategory>', 'std::vector<RawStringFormat>',
            'std::vector<std::pair<std::string, unsigned>>'
        ]
        if is_known_type:
            return True
        elif '::' in optiontype:
            # An attempt at future-proofing this code...
            unknown_optiontypes.append(optiontype)
            return True
        return False

    options = dumpformatoptions.read_options(format_h_lines, isknownoptiontype)
    try:
        options += dumpformatoptions.read_options(includestyle_h, isknownoptiontype)
    except Exception as exc:
        pass
    options = sorted(options, key=lambda x: x.name)
    return options, unknown_optiontypes


def parse_styles(clangworkdir):
    """Returns a list of style definitions for every commit of Format.h in the
     following style, this example is abbreviated and modified for clarity.
    [
    (u'3.3',
     [u'LLVM', u'Google', u'Chromium', u'Mozilla'],
     [('AccessModifierOffset', ('int', [])),
      ('PointerBindsToType', ('bool', []))]),
    (u'3.4',
     [u'LLVM', u'Google', u'Chromium', u'Mozilla', u'Predefined'],
     [('AccessModifierOffset', ('int', [])),
      ('BreakBeforeBraces', ('BraceBreakingStyle', [u'Linux', u'Allman'])),
      ('PointerBindsToType', ('bool', []))])
    ]
    """
    unknown_types = set()

    style_versions = []
    for commit, format_h, includestyle_h, conf_py in git_format_commits(clangworkdir):
        base_formats = []
        release = commit
        # Use the clang version number instead of the commithash
        # if it can be found in clangs conf.py.
        for line in conf_py.splitlines():
            m = re.match("release = '(.*)'", line)
            if m:
                release = m.group(1)

        format_h_lines = format_h.splitlines()
        # Record the format style names
        # e.g. 'FormatStyle getChromiumStyle();' => 'Chromium'
        for line in format_h_lines:
            m = re.match('\s*FormatStyle\s*get(\w+)Style\([^\)]*\);\s*', line)
            if m:
                formatname = m.group(1)
                if formatname != 'No':
                    # NoStyle is not a style named No.
                    base_formats.append(formatname)

        try:
            options, unknown_optiontypes = parse_options(format_h_lines, includestyle_h)
        except Exception:
            continue
        for t in unknown_optiontypes:
            unknown_types.add(t)
        style_options = []
        for opt in options:
            configs = []
            if opt.enum:
                for enumvalue in opt.enum.values:
                    configs.append(re.sub('.*_', '', enumvalue.name))
            elif opt.nested_struct:
                for nestedoption in opt.nested_struct.values:
                    configs.append(nestedoption.name)
            style_options.append((opt.name, (opt.type, configs)))
        style_versions.append((release, base_formats, style_options))

    return style_versions


def generate_style_history(clangworkdir):
    """Prints the style available style configurations
    following the commit history, e.g.:
    # Clang 3.4
    + BasedOnStyle string
            LLVM
            WebKit
    + SpaceAfterControlStatementKeyword bool
    # Clang 3.4
    + TabWidth unsigned
            Never
            ForIndentation
            Always
    # Clang 3.5
    - SpaceAfterControlStatementKeyword bool
    + AllowShortFunctionsOnASingleLine bool
    """
    prev_styles = OrderedDict([])
    changelines = []
    unknown_types = set()

    style_versions = parse_styles(clangworkdir)
    if not style_versions:
        return
    _, latest_base_formats, _ = style_versions[-1]

    supported = FIRST_SUPPORTED_VERSION is None
    for release, base_formats, options in style_versions:
        if release == FIRST_SUPPORTED_VERSION:
            supported = True
        # To keep the history small skip ancient clang versions.
        if not supported:
            continue

        # In old clang-format versions only use base format styles that
        # are still supported in the most recent version.
        base_formats = [b for b in base_formats if b in latest_base_formats]
        options = [('BasedOnStyle', ('string', base_formats))] + options
        options = OrderedDict(options)
        actions = []
        # Delete options that no longer exist since this commit
        for optionname, optiondef in prev_styles.items():
            if optionname not in options:
                actions.append("- %s" % option_rep(optionname, optiondef))

        # Add new options from this commit
        for optionname, optiondef in options.items():
            oldstyledef = prev_styles.get(optionname)
            if optiondef != oldstyledef:
                actions.append("+ %s" % option_rep(optionname, optiondef))
        if actions:
            changelines.append("# Clang " + release)
            changelines.extend(actions)
        prev_styles = options

    outline("\n".join(changelines))
    sys.stdout.flush()
    if unknown_types:
        reportwarning("\nWarning: Not all of the encountered option types could be"
                      " recognized"
                      "\n         while building the history of known clang-format"
                      " options."
                      "\n         This should not be a problem but here they are:")
        for u in unknown_types:
            reportwarning("Unknown option type: %s" % u)


def option_rep(optionname, optiondef):
    """Returns a textual representation of an option.

    option_rep('IndentCaseLabels', ('bool', []))
    => 'IndentCaseLabels bool'

    option_rep('PointerAlignment', ('PointerAlignmentStyle',
               [u'Left', u'Right', u'Middle']))
    => 'PointerAlignment PointerAlignmentStyle
               Left
               Right
               Middle'
    """
    optiontype, configs = optiondef
    fragments = [optionname + ' ' + optiontype]
    for c in configs:
        fragments.append(" " * 8 + c)
    rep = "\n".join(fragments)
    return rep


def main():
    parser = argparse.ArgumentParser(
        description='Create clang-format format history from a clang git repo.')
    parser.add_argument('clangrepo', nargs=1, help='path of a clang repository')
    args = parser.parse_args()
    generate_style_history(args.clangrepo[0])


if __name__ == '__main__':
    if PY3:
        sys.stdout = codecs.getwriter('utf8')(sys.stdout.buffer)
        sys.stderr = codecs.getwriter('utf8')(sys.stderr.buffer)
    sys.exit(main())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# This script runs whatstyle on a number of source code
# examples and saves the resulting style and the reformatted
# source to a result directory. The metric of the style is also
# saved and compared to the last known good metric.
# The slightest difference might be caused by an error, a performance
# improvement because the same style was found earlier or a change
# of the formatter version string.
#
# This script is used for regression testing in a virtual machine defined
# in Vagrantfile because the metric comparison requires exactly the same
# formatter versions.
#
# When running these example tests on a machine where different results
# expected, this sequence can be used from a checked out whatstyle git repo:
#   $ ./tools/remove_resultdirs.py   # this removes the last known good results
#   $ python tests/test_examples.py
#   $ git diff                       # look at the differences
#
# Testing only some formatters can be as follows:
#   $ WHATSTYLE_TEST_FORMATTERS="uncrustify astyle" \
#       python tests/test_examples.py ExamplesTestCase.test_xv6

from __future__ import print_function

import sys

if (((sys.version_info[0] == 2) and (sys.version_info[1] < 7)) or (
    (sys.version_info[0] == 3) and (sys.version_info[1] < 2))):
    sys.stderr.write('Error: Python 2.7 or when running on Python 3 at least Python 3.2'
                     ' is required to run whatstyle\n')
    sys.exit(1)

import errno
import logging
import os
import subprocess
import unittest

from os.path import abspath, basename, dirname, exists, join

# Set up the Python path so we find the whatstyle module in the
# parent directory relative to this file.
sys.path.insert(1, dirname(dirname(abspath(__file__))))

import whatstyle
from whatstyle import sysfilename, unifilename, unistr, bytestr

PY2 = sys.version_info[0] == 2
PY3 = sys.version_info[0] == 3

if PY2:
    text_type = unicode
    binary_type = str
else:
    text_type = str

WHATSTYLE_VIA_SUBPROCESS = False

logging.basicConfig(stream=sys.stdout)
logging.getLogger().setLevel(logging.DEBUG)

# ---------------------------------------------------------------------


def here():
    return dirname(abspath(unifilename(__file__)))


def exampledir():
    return join(here(), 'examples')


def setUpModule():
    pass


def tearDownModule():
    pass

# ---------------------------------------------------------------------

CURRENT_TEST = None


class ExamplesTestCase(unittest.TestCase):

    maxDiff = None

    def setUp(self):
        global CURRENT_TEST
        CURRENT_TEST = self.id()

    def test_python(self):
        self.assertMultiLineEqual('', run_spec({'files': [
            'bottle_sqlalchemy/bottle_sqlalchemy.py'
        ]}))

    def test_html(self):
        self.assertMultiLineEqual('', run_spec({'files': ['foundation6/index.html']}))

    def test_gumbo(self):
        self.assertMultiLineEqual('', run_spec({'files': ['gumbo-parser/utf8.c'],
                                                'formatters':
                                                ['clang-format', 'uncrustify']}))

    def test_objc(self):
        self.assertMultiLineEqual('', run_spec(
            {'files': ['HexFiend/HFIndexSet.h', 'HexFiend/HFIndexSet.m'],
             'formatters': ['clang-format', 'uncrustify']}))

    def test_javascript(self):
        self.assertMultiLineEqual('', run_spec({'files': ['jasmine/Clock.js'],
                                                'formatters': ['clang-format']}))

    def test_resilient_cpp(self):
        self.assertMultiLineEqual('', run_spec({'files': ['lodepng/pngdetail.cpp'],
                                                'formatters': ['clang-format', 'indent'],
                                                'mode': 'resilient'}))

    def test_international(self):
        self.assertMultiLineEqual('', run_spec({'files': ['international/IŋƫƐrnætiønæl.c']}))

    def test_resilient_c(self):
        self.assertMultiLineEqual('', run_spec({'files': ['lua/lstate.h', 'lua/lstate.c'],
                                                'mode': 'resilient'}))

    def test_nginx(self):
        self.assertMultiLineEqual('', run_spec({'files':
                                                ['nginx/ngx_buf.h', 'nginx/ngx_buf.c'],
                                                'mode': 'resilient'}))

    def test_java(self):
        self.assertMultiLineEqual('',
                                  run_spec({'files': ['maven/ProjectModelResolver.java']}))

    def test_stylediff_noalign(self):
        self.assertMultiLineEqual('', run_spec(
            {'files': ['ref_no_align/noalign.c', 'ref_no_align/noalign_ref.c'],
             'mode': 'stylediff',
             'references': True}))

    def test_stylediff_space_brace(self):
        self.assertMultiLineEqual('', run_spec(
            {'files': ['ref_space_brace/hello.c', 'ref_space_brace/hello_ref.c'],
             'mode': 'stylediff',
             'references': True}))

    def test_xv6(self):
        self.assertMultiLineEqual('', run_spec({'files': ['xv6/printf.c']}))

    def test_scala(self):
        filename = 'spark/SparkLR.scala'
        fmts = whatstyle.choose_formatters([filename], report=False)
        if not fmts:
            self.skipTest('no formatters for scala installed')
            return
        ignore = '--ignoreopts=alignSingleLineCaseStatements.maxArrowIndent'
        self.assertMultiLineEqual('', run_spec({'files': [filename], 'extraargs': [ignore]}))

    def test_rfmt(self):
        filename = 'rfmt/test.R'
        fmts = whatstyle.choose_formatters([filename], report=False)
        if not fmts:
            self.skipTest('no formatters for R installed')
            return
        # Use a very short column limit to give the cost functions something to work with.
        shortlines = '--startstyle={margin1: 30}'
        self.assertMultiLineEqual('', run_spec({'files': [filename],
                                                'extraargs': [shortlines]}))

    def test_polycode(self):
        filename = 'polydraw/intersection.rs'
        fmts = whatstyle.choose_formatters([filename], report=False)
        if not fmts:
            self.skipTest('no formatters for Rust installed')
            return
        self.assertMultiLineEqual('', run_spec({'files': [filename]}))

    def test_rustfmt(self):
        filename = 'rustfmt/checkstyle.rs'
        fmts = whatstyle.choose_formatters([filename], report=False)
        if not fmts:
            self.skipTest('no formatters for Rust installed')
            return
        self.assertMultiLineEqual('', run_spec({'files': [filename]}))

    def tearDown(self):
        global CURRENT_TEST
        CURRENT_TEST = None

# ---------------------------------------------------------------------


def logger():
    return logging.getLogger(CURRENT_TEST)


def files_equal(refdir, testfiles):
    diffs = []
    for f in testfiles:
        base = basename(f)
        reffile = join(refdir, base)
        if not os.path.exists(reffile):
            diffs.append("# Reference file %s does not exist" % reffile)
            continue
        if not os.path.exists(f):
            diffs.append("# Result file %s does not exist" % reffile)
            continue
        with open(f, 'rb') as fp:
            data = fp.read()
            exit_code, diff = whatstyle.compute_unified_diff(reffile,
                                                             data,
                                                             fromfile=reffile,
                                                             tofile=f,
                                                             n=2)
            diff = list(diff)
            if not diff:
                continue
            metricfile = whatstyle.metricfile(reffile)
            if metricfile == reffile:
                with open(reffile, 'rb') as fp:
                    data = bytestr(data)
                    refdata = fp.read()
                    commentpos = refdata.find(b'#')
                    if commentpos >= 0 and data[:commentpos] == refdata[:commentpos]:
                        # Only the formatter name or version is different and that
                        # is no reason to fail the test.
                        continue
            diffs.extend(diff)
    return diffs


def run_spec(spec):
    inputfiles = []
    for rp in spec['files']:
        rp = unistr(rp)
        components = rp.split('/')
        inputfile = join(exampledir(), *components)
        if not exists(inputfile):
            print('Skipping %s which does not exist.' % inputfile)
            continue
        inputfiles.append(inputfile)

    selected_formatters = os.getenv('WHATSTYLE_TEST_FORMATTERS', '').split()

    inputfiles = [unifilename(f) for f in inputfiles]
    exdir = dirname(inputfiles[0])
    formatters = whatstyle.choose_formatters(inputfiles)
    handled_formatters = set()
    alldiffs = []
    for formatter in formatters:
        fmtbasename = basename(os.path.splitext(formatter)[0])
        fmtclass = whatstyle.formatterclass(fmtbasename)
        fmtname = fmtclass.shortname
        if selected_formatters and fmtname not in selected_formatters:
            continue
        spec_fmts = spec.get('formatters')
        if spec_fmts is not None and fmtname not in spec_fmts:
            continue
        if fmtname in handled_formatters:
            # Do not run the same formatter class several times.
            continue
        else:
            handled_formatters.add(fmtname)
        resultdir = join(exdir, unifilename('result_%s' % fmtname))
        if os.path.isdir(resultdir):
            new_resultdir = unifilename(resultdir) + unifilename('_test')
        else:
            new_resultdir = resultdir
        try:
            os.makedirs(new_resultdir)
        except OSError as exc:
            if exc.errno != errno.EEXIST:
                raise
        resultfiles = run_whatstyle(formatter, spec, inputfiles, new_resultdir)
        if resultdir == new_resultdir:
            continue
        # Compare results against the test reference files
        diffs = files_equal(resultdir, resultfiles)
        if not diffs:
            for f in resultfiles:
                os.remove(f)
            os.rmdir(new_resultdir)
        else:
            alldiffs.append('# Formatter: %s\n' % formatter)
            alldiffs.extend(diffs)
    return ''.join(alldiffs)

# ---------------------------------------------------------------------


def run_whatstyle(formatter, spec, inputfiles, resultdir):
    prog = abspath(join(here(), '..', 'whatstyle.py'))
    fmtname = basename(formatter)
    fmtclass = whatstyle.formatterclass(fmtname)
    fmtinst = fmtclass(formatter)
    stylefile = fmtclass.configfilename

    configfile = join(resultdir, stylefile)

    cmdargs = [
        prog,
        # '-v', '-v', '-v',
        # '--cache=off',
        # '--concurrent=off',
        # '--showdiff',
        # '--color=off',
        # '--debug=all',
        # '--info=all',
        '--noprogress',
        '--difftool=difflib',
        '--formatter',
        formatter,
        '--savemetric',
        '--output',
        configfile
    ]
    references = spec.get('references')
    if references:
        cmdargs.append('--references')
    mode = spec.get('mode')
    if mode:
        cmdargs.append('--mode')
        cmdargs.append(mode)
    extraargs = spec.get('extraargs')
    if extraargs:
        cmdargs.extend(extraargs)

    cmdargs.extend(inputfiles)
    if WHATSTYLE_VIA_SUBPROCESS:
        proc = subprocess.Popen(cmdargs)
        proc.communicate()
        returncode = proc.returncode
    else:
        cmdargs = cmdargs[1:]
        parser = whatstyle.cmdline_parser()
        args = parser.parse_args(cmdargs)
        returncode = whatstyle.run_with_args(args, parser)

    resultfiles = [configfile, whatstyle.metricfile(configfile)]
    if references:
        resultfiles.append(configfile + '_diff')
    srcs, _ = whatstyle.split_reffiles(references, inputfiles)
    for src in srcs:
        inputbase = basename(src)
        # Copy the input source to the result directory
        sourceoutput = join(resultdir, inputbase)
        resultfiles.append(sourceoutput)
        try:
            os.remove(sourceoutput)
        except OSError as exc:
            if exc.errno != errno.ENOENT:
                raise
        if returncode == 0:
            fmtinst.reformat(src, sourceoutput, configfile)
    return resultfiles


def main():
    unittest.main()


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# This script opens a 3-way file comparison of files of which one is pulled out of git.
#
# If for example the argument is
# tests/examples/foundation6/result_tidy/index.html, then:
#
# 1) tests/examples/foundation6/index.html is the basefile,
# 2) the latest version of the argument in git is the second file to compare and
# 3) the argument itself is the third file to compare.
#
# The option '--results' opens diffs for all modified source code result files.
#
# The results from test_examples.py are present in the git repo.
# When changing formatters, algorithms or diff metrics we can run remove_resultdirs.py
# and then test_examples.py.
# Then we can use this script to get an overview if the changes have influenced the
# reformatted source code test files for better or worse by looking at the 3-way diff
# between the original source, the previous test result and the new test result.
# The merge conflict hunks show how the test results differ in context with the original.

from __future__ import print_function

import sys

if (((sys.version_info[0] == 2) and (sys.version_info[1] < 7)) or (
    (sys.version_info[0] == 3) and (sys.version_info[1] < 2))):
    sys.stderr.write('Error: Python 2.7 or when running on Python 3 at least Python 3.2'
                     ' is required to run whatstyle\n')
    sys.exit(1)

import argparse
import threading
import os
import subprocess
import tempfile

from collections import namedtuple
from os.path import abspath, dirname, basename, join

DIFF3_VARNAME = 'WHATSTYLE_DIFF3'

PY2 = sys.version_info[0] == 2
PY3 = sys.version_info[0] == 3

if PY2:
    text_type = unicode
    binary_type = str
else:
    text_type = str
    binary_type = bytes


def unistr(text):
    if not isinstance(text, text_type):
        text = text.decode('utf-8')
    return text


def bytestr(text):
    if not isinstance(text, binary_type):
        text = text.encode('utf-8')
    return text


def sysfilename(filename):
    if not isinstance(filename, text_type):
        return filename
    try:
        return filename.encode(sys.getfilesystemencoding())
    except UnicodeEncodeError:
        return filename.encode('utf-8')


ExeResult = namedtuple('ExeResult', ['returncode', 'stdout', 'stderr'])


def call_executable(executable, cmdargs):
    cmdargs = [sysfilename(x) for x in cmdargs]
    subprocess.call([executable] + cmdargs)


def run_executable(executable, cmdargs, stdindata=None):
    cmdargs = [sysfilename(x) for x in cmdargs]
    proc = subprocess.Popen([executable] + cmdargs,
                            stdin=subprocess.PIPE,
                            stdout=subprocess.PIPE,
                            stderr=subprocess.PIPE)
    stdout, stderr = proc.communicate(input=stdindata)
    return ExeResult(proc.returncode, stdout, stderr)


def gitcmd(cmdargs):
    return run_executable('git', cmdargs)


def gitcat(cwd, commit, filename):
    """Returns the file content for a commit.
    """
    ret, stdout, stderr = gitcmd(['-C', cwd, 'show', '%s:%s' % (commit, unistr(filename))])
    return stdout


def git_commits(cwd, relpath):
    """Return a list of commit hashes for relpath sorted from newest to oldest.
    """
    ret, stdout, stderr = gitcmd(['-C', cwd, 'log', '--pretty=oneline', relpath])
    lines = unistr(stdout).splitlines()
    return list([l.split()[0] for l in lines])


def open_for_filename(tool, filename):
    """Creates a temporary file from a git revision of a result file and runs the diff
    tool with it and the parent and other sibling file.
    """
    base = basename(filename)
    parts = [base]
    workdir = dirname(filename)
    firstworkdir = workdir
    prev = None
    while workdir and workdir != prev:
        if os.path.isdir(join(workdir, b'.git')):
            break
        parts.insert(0, basename(workdir))
        prev = workdir
        workdir = dirname(workdir)
    else:
        print('error: did not find .git in %s or its parent directories' % firstworkdir)
        return 2

    relpath = bytestr(os.sep).join(parts)
    commits = git_commits(workdir, relpath)
    content = gitcat(workdir, commits[0], relpath)
    td = sysfilename(tempfile.gettempdir())
    tmpfile = os.path.join(td, b'whatstyle_3gdiff_' + base)
    with open(tmpfile, 'wb') as fp:
        fp.write(content)

    file1 = join(dirname(dirname(filename)), base)
    file2 = tmpfile
    file3 = filename

    t = threading.Thread(target=call_executable, args=[tool, [file1, file2, file3]])
    t.start()
    return t


def modified_results(tool):
    workdir = dirname(dirname(abspath(__file__)))
    exeresult = run_executable('git', ['-C', workdir, 'ls-files', '-z', '-m',
                                       'tests/examples/*/result_*'])
    files = exeresult.stdout.split(b'\x00')
    ignoreexts = bytestr('.cfg .cfg_diff .conf .pro .pro_diff .txt').split()
    result_filenames = []
    for f in files:
        if not f:
            continue
        filename = join(bytestr(workdir), f)
        _, ext = os.path.splitext(filename)
        if not ext or ext in ignoreexts:
            continue
        if not os.path.exists(filename):
            continue
        result_filenames.append(filename)
    diff_for_files(tool, result_filenames)


def diff_for_files(tool, filenames):
    threads = [open_for_filename(tool, f) for f in filenames]
    for t in threads:
        t.join()


def main():
    parser = argparse.ArgumentParser(description='Open a diff tool with three files')
    parser.add_argument('filename', nargs='*', help='input filename')
    # The info and debug options were inspired by rsync.
    parser.add_argument('--tool', help='path of diff tool')
    parser.add_argument('--results', action='store_true', help='show results diffs')

    args = parser.parse_args()
    tool = args.tool
    if tool is None:
        tool = os.environ.get(DIFF3_VARNAME)
    if tool is None:
        parser.error('Please specify the 3-way file comparison tool with --tool or set %s' %
                     DIFF3_VARNAME)
    if args.results:
        return modified_results(tool)
    filenames = args.filename
    if not filenames:
        parser.error('Please specify at least one source file')

    filenames = [os.path.abspath(f) for f in filenames]
    diff_for_files(tool, filenames)
    return 0


if __name__ == '__main__':
    sys.exit(main())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# This script extracts program crashes from a dump file that was recorded with:
# whatstyle.py --color on --keeptempfiles --debug popenio ...
# The "--color on" is necessary so that it can be distinguished if a space character
# seperates two arguments or is part of the argument itself.
#
# Running 'outtakes.py < dump.txt' will create nested directories like
# outtakes/c_-4_396 that contain the returncode, standard input, output and error and
# an eventually a configfile from the dump.
# It should be sufficient to enter one of these directories and call ./callfmt.sh
# to reproduce a crash.

from __future__ import print_function

import sys

if (((sys.version_info[0] == 2) and (sys.version_info[1] < 7)) or (
    (sys.version_info[0] == 3) and (sys.version_info[1] < 2))):
    sys.stderr.write('Error: Python 2.7 or when running on Python 3 at least Python 3.2'
                     ' is required to run whatstyle\n')
    sys.exit(1)

import argparse
import errno
import os
import re
import shutil
try:
    from shlex import quote as shellquote
except ImportError:
    from pipes import quote as shellquote

from collections import Counter

PY2 = sys.version_info[0] == 2
PY3 = sys.version_info[0] == 3

if PY2:
    text_type = unicode
    binary_type = str
else:
    text_type = str
    binary_type = bytes

DESTDIR = 'outtakes'

HAS_FSCODEC = hasattr(os, 'fsdecode')


def unistr(text, errors='strict'):
    if isinstance(text, text_type):
        return text
    try:
        return text.decode('utf-8', errors=errors)
    except UnicodeDecodeError:
        if HAS_FSCODEC:
            return os.fsdecode(text)
        raise


def bytestr(text):
    if isinstance(text, binary_type):
        return text
    try:
        return text.encode('utf-8')
    except UnicodeEncodeError:
        if HAS_FSCODEC:
            return os.fsencode(text)
        raise


def rawstream(fp):
    if PY3:
        try:
            return fp.buffer
        except AttributeError:
            # There might be a BytesIO behind fp.
            pass
    return fp


def write(s, fp=None):
    """Write s to the binary stream fp (default is stdout).
    """
    efp = fp if fp is not None else sys.stdout
    rawstream(efp).write(bytestr(s))


def outline(s=b'', end=b'\n', fp=None):
    write(bytestr(s) + bytestr(end), fp=fp)


re_ansi = re.compile(br'\x1b\[(?:\d*(?:;\d+)*)m')


def extract_outtakes(filename, maxpercode=0):
    re_lengths = re.compile(br'debug_popen: len\(stdin\):(\d+) => returncode:(-?\d+)'
                            br' len\(stdout\):(\d+) len\(stderr\):(\d+)')
    valid_retcodes = set(range(100))
    retcounters = Counter()
    with open(filename, 'rb') as fp:
        count = 0
        cfgfile = None
        command = None
        while True:
            line = fp.readline()
            if not line:
                break
            prefix = b'debug_popen: '
            m = re_lengths.match(line)
            if m:
                lin, retcode, lout, lerr = [int(x) for x in m.groups()]
                if retcode in valid_retcodes:
                    continue
                if 1 <= maxpercode <= retcounters[retcode]:
                    continue
                retcounters[retcode] += 1
                otdir = os.path.join(DESTDIR, "c_%s_%s" % (str(retcode), str(count)))
                try:
                    os.makedirs(otdir)
                except OSError as exc:
                    if exc.errno != errno.EEXIST:
                        raise
                if command:
                    fname = os.path.join(otdir, 'callfmt.sh')
                    with open(fname, 'wb') as cfg:
                        if lin > 0:
                            command = command + b' < stdin.txt'
                        cfg.write(command + b'\n')
                    os.chmod(fname, 0o755)
                    command = None
                with open(os.path.join(otdir, 'retcode.txt'), 'wb') as cfg:
                    cfg.write(bytestr(str(retcode)) + b'\n')
                if cfgfile:
                    shutil.copyfile(cfgfile, os.path.join(otdir, os.path.basename(cfgfile)))
                    cfgfile = None
                for chan, chanlen in zip(['stdin', 'stdout', 'stderr'], [lin, lout, lerr]):
                    if chanlen == 0:
                        continue
                    line = fp.readline()
                    if line == b'debug_popenio: ' + bytestr(chan) + b':"""\\\n':
                        data = fp.read(chanlen)
                        with open(os.path.join(otdir, '%s.txt' % chan), 'wb') as cfp:
                            cfp.write(data)
                        fp.readline()  # This should be """
            elif line.startswith(prefix):
                line = line[len(prefix):]
                line = line.rstrip(b'\r\n')
                args = re_ansi.split(line)
                cmdargs = []
                if len(args) > 1 and not args[0] and not args[-1]:
                    for idx, arg in enumerate(args[1:-1]):
                        if idx % 2 == 1:
                            if arg == b' ':
                                continue
                            else:
                                write(b"Unexpected debug_popen line: " + line, fp=sys.stderr)
                        uarg = arg.decode('raw-unicode-escape')
                        if idx > 0 and os.path.abspath(arg) and os.path.isfile(arg):
                            cfgfile = uarg
                            uarg = os.path.basename(uarg)
                        cmdargs.append(shellquote(uarg).encode('raw-unicode-escape'))
                if cmdargs:
                    command = b' '.join(cmdargs)
                    count += 1
    return 0


def main():
    parser = argparse.ArgumentParser(description='Extract data from formatter crashes')
    parser.add_argument('filename', help='input dump filename')
    parser.add_argument('--maxpercode',
                        type=int,
                        default=-1,
                        help='only extract this many calls per returncode\n'
                        ' 0 means unlimited (default: 10)')
    args = parser.parse_args()

    if not args.filename:
        parser.error('Please specify the input dump filename')

    return extract_outtakes(args.filename, maxpercode=args.maxpercode)


if __name__ == '__main__':
    sys.exit(main())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 Michael Krause ( http://krause-software.com/ ).

# You are free to use this code under the MIT license:
# http://opensource.org/licenses/MIT
"""Tests for whatstyle."""

from __future__ import print_function

import argparse
import json
import os
import re
import sys
import tempfile
import unittest
from collections import OrderedDict
from io import BytesIO
from os.path import abspath, dirname
from pprint import pprint

# Set up the Python path so we find the whatstyle module in the
# parent directory relative to this file.
sys.path.insert(1, dirname(dirname(abspath(__file__))))

import whatstyle
from whatstyle import bytestr, unistr


def run_args(*args):
    outbuf, errbuf = BytesIO(), BytesIO()
    with RedirectStdStreams(stdout=outbuf, stderr=errbuf):
        parser = whatstyle.cmdline_parser(ErrorRecordingArgumentParser)
        args = parser.parse_args(*args)
        ret = whatstyle.run_with_args(args, parser)
    return ret, outbuf.getvalue(), errbuf.getvalue()

# ---------------------------------------------------------------
# This is RedirectStdStreams from an answer by Rob Cowie on
# http://stackoverflow.com/questions/6796492/python-temporarily-redirect-stdout-stderr/6796752#6796752
# which is used to record the output of whatstyle as if called from the
# command line.


class RedirectStdStreams(object):

    def __init__(self, stdout=None, stderr=None):
        self._stdout = stdout or sys.stdout
        self._stderr = stderr or sys.stderr

    def __enter__(self):
        self.old_stdout, self.old_stderr = sys.stdout, sys.stderr
        self.old_stdout.flush()
        self.old_stderr.flush()
        sys.stdout, sys.stderr = self._stdout, self._stderr

    def __exit__(self, exc_type, exc_value, traceback):
        self._stdout.flush()
        self._stderr.flush()
        sys.stdout = self.old_stdout
        sys.stderr = self.old_stderr

# ---------------------------------------------------------------


class ErrorRecordingArgumentParser(argparse.ArgumentParser):

    def __init__(self, *args, **kwargs):
        super(ErrorRecordingArgumentParser, self).__init__(*args, **kwargs)
        self.testerrors = []

    def error(self, message):
        self.testerrors.append(message)

# ---------------------------------------------------------------


def tmpfile_with_data(data, **kwargs):
    fd, filename = tempfile.mkstemp(**kwargs)
    os.write(fd, bytestr(data))
    os.close(fd)
    return filename


def defaultcache():
    if whatstyle.sqlite3:
        return whatstyle.Cache(whatstyle.default_cachepath() + '.sqlite')
    return None


class ParsehistoryTestCase(unittest.TestCase):

    def test_tabcombo(self):
        history = whatstyle.parse_style_history()
        self.assertEqual(len(history), 62)


class DiffTestCase(unittest.TestCase):

    def setUp(self):
        self.cache = defaultcache()

    def test_builtin_diff(self):
        source = "a\nb\nc\n"
        content2 = bytestr("a\ne\nc\n")
        bn = "whatstyle_diff_"
        filename = tmpfile_with_data(source, suffix=bn)
        try:
            code, diff = whatstyle.compute_unified_diff(filename, content2)
            diffdata = ''.join(diff)
            diffdata = bytestr(diffdata)
            metric = whatstyle.metric_for_mindiff(diffdata)
            self.assertEqual(metric, 4)
        finally:
            os.remove(filename)

    def test_pathological_diffs(self):
        """Assure that a diff of a file with a different indentation or
         a few added lines has a better diff metric than a file that
         is basically empty.
        """
        difftool = whatstyle.BUILTIN_DIFF
        origlines = []
        newlines = []
        # The original file has 200 lines with numbers and a 4 space indent.
        for i in range(1, 201):
            origlines.append('    %d' % i)
            # The file to compare with has a 2 space indent.
            newlines.append('  %d' % i)
        source = '\n'.join(origlines) + '\n'
        mod1 = '\n'.join(newlines) + '\n'
        mod2 = mod1 + '\n\n\n\n\n\n\n\n'
        mod3 = ''
        mod4 = '\n'

        fd, tmpfilename = tempfile.mkstemp(suffix='.c', prefix='whatstyle_test_')
        os.write(fd, bytestr(source))
        os.close(fd)
        try:
            dists = whatstyle.distances_from_diffs(difftool,
                                                   [(tmpfilename, mod1), (tmpfilename, mod2),
                                                    (tmpfilename, mod3), (tmpfilename, mod4)])
            dists = list(dists)
            self.assertLess(dists[0], dists[2])
            self.assertLess(dists[0], dists[3])
            self.assertLess(dists[1], dists[2])
            self.assertLess(dists[1], dists[3])
        finally:
            os.remove(tmpfilename)

    def tearDown(self):
        if self.cache is not None:
            self.cache.close()


class CmdlineTestCase(unittest.TestCase):

    source = """\
def func():

  def nestedfunc(): # type: () -> None
    pass

  print("Hello World!")
"""

    expected = """\
### This style was chosen for your .style.yapf - it perfectly matches your sources.
[style]
based_on_style = chromium
spaces_before_comment = 1
"""

    def test_convert(self):
        lines = self.source.splitlines()
        data = os.linesep.join(lines) + os.linesep
        filename = tmpfile_with_data(data, suffix='.py', prefix='whatstyle_')
        try:
            ret, outtxt, errtxt = run_args(['--quiet', '--color=off', '--stderr', filename])
            self.assertEqual(ret, 0)
            self.assertEqual(outtxt, bytestr(self.expected))
        finally:
            os.remove(filename)


NESTED_VARS_DUMP = """\
---
Language:        Cpp
# BasedOnStyle:  LLVM
AlignTrailingComments: true
DerivePointerAlignment: false
PointerAlignment: Right
TabWidth:        8
UseTab:          Never
...

"""

JQUERY_TABS = """\
jQuery.noConflict = function(deep) {
\treturn jQuery;
};
"""


class VariantTestCase(unittest.TestCase):

    def setUp(self):
        pass

    def test_quoting(self):
        a = bytestr(r'\x41\u0042\U000043')
        b = whatstyle.surrdecode(a)
        b = whatstyle.escape_illegal_xmlchars(b)
        b = whatstyle.unescape_illegal_xmlchars(b)
        b = whatstyle.surrencode(b)
        self.assertEqual(a, b)

    def test_tableiter(self):
        bindata = []
        text = 'IŋƫƐrnætiønæl'
        text_a = bytestr(text) + b'\0\1\2\n'
        text_b = bytestr(r'\x41\u0042\U000043')
        for i in range(256):
            fragments = [bytestr('%0x %d ' % (i, i))]
            h = int(i / 256)
            l = i % 256
            if h:
                fragments.append(whatstyle.int2byte(h))
            fragments.append(whatstyle.int2byte(l))
            data = b''.join(fragments)
            udata = data.decode('utf-8', 'replace')
            ulen = whatstyle.unilen(udata)
            data = b' ' * (16 - ulen) + data + b'\n'
            bindata.append(data)
        text_b += b''.join(bindata)
        try:
            bodies = []
            for tablestart, tbody, tableend in whatstyle.table_iter([(text_a, text_b)],
                                                                    1,
                                                                    fromdesc='',
                                                                    todesc=''):
                bodies.append(tbody)
            html = '\n'.join(bodies)
            buf = BytesIO()
            html = '<html><body><table>' + html + '</table></body></html>'
            whatstyle.htmldiff2ansi(html, enc='utf-8', fp=buf)
            ansi = buf.getvalue()
            # whatstyle.outline(ansi)
            self.assertEqual(len(re.findall(bytestr(text) + b'\0\1\2', ansi)), 1)
        except whatstyle.ETree.ParseError:
            raise

    def tearDown(self):
        pass


class OptioncomboTestCase(unittest.TestCase):

    def setUp(self):
        clangformat = 'clang-format'
        self.cache = defaultcache()
        formatters = whatstyle.installed_formatters([clangformat])
        if not formatters:
            self.fail('%s is not installed' % clangformat)
        self.formatter = whatstyle.ClangFormatter(clangformat, cache=self.cache)
        self.formatter.register_options()
        self.assertIsNotNone(self.formatter.styledefinition)

    def test_bools(self):
        option = whatstyle.option_make('IndentCaseLabels', 'bool', ())
        result = self.formatter.variants_for(option)
        self.assertEqual(result, [{'IndentCaseLabels': True}, {'IndentCaseLabels': False}])

    def test_tabcombo(self):
        v = self.formatter.variants_for
        res = v(whatstyle.option_make('UseTab', 'UseTabStyle', ('Never', 'ForIndentation',
                                                                'Always')))
        # yapf: disable
        self.assertEqual(
            res, [{'UseTab': 'Never'},
                  {'UseTab': 'ForIndentation', 'TabWidth': 1},
                  {'UseTab': 'ForIndentation', 'TabWidth': 2},
                  {'UseTab': 'ForIndentation', 'TabWidth': 3},
                  {'UseTab': 'ForIndentation', 'TabWidth': 4},
                  {'UseTab': 'ForIndentation', 'TabWidth': 5},
                  {'UseTab': 'ForIndentation', 'TabWidth': 6},
                  {'UseTab': 'ForIndentation', 'TabWidth': 7},
                  {'UseTab': 'ForIndentation', 'TabWidth': 8},
                  {'UseTab': 'Always', 'TabWidth': 1},
                  {'UseTab': 'Always', 'TabWidth': 2},
                  {'UseTab': 'Always', 'TabWidth': 3},
                  {'UseTab': 'Always', 'TabWidth': 4},
                  {'UseTab': 'Always', 'TabWidth': 5},
                  {'UseTab': 'Always', 'TabWidth': 6},
                  {'UseTab': 'Always', 'TabWidth': 7},
                  {'UseTab': 'Always', 'TabWidth': 8}])
        # yapf: enable

    def test_nestedoption(self):
        style = self.formatter.styledefinition
        option = whatstyle.styledef_option(style, 'BraceWrapping')
        if option is None:
            # This clang-format is probably too old.
            return
        res = self.formatter.variants_for(option)
        # We expect 11 nested options with true and false values.
        self.assertEqual(len(res), 22)

    def tearDown(self):
        if self.cache is not None:
            self.cache.close()


class StylecompareTestCase(unittest.TestCase):

    maxDiff = None
    jenc = json.JSONEncoder(sort_keys=True).encode

    def setUp(self):
        clangformat = 'clang-format'
        self.cache = defaultcache()
        formatters = whatstyle.installed_formatters([clangformat])
        if not formatters:
            self.fail('%s is not installed' % clangformat)
        self.formatter = whatstyle.ClangFormatter(clangformat, cache=self.cache)
        self.formatter.register_options()
        self.assertIsNotNone(self.formatter.styledefinition)

    def test_enum(self):
        estyle = self.formatter.effective_style(whatstyle.style_make())
        self.assertEqual(estyle.get('UseTab'), 'Never')

    def test_yaml(self):
        res = whatstyle.parse_miniyaml("""\
---
Language:        Cpp
# BasedOnStyle:  LLVM
...
""")
        self.assertEqual(res, OrderedDict([('Language', 'Cpp'), ('BasedOnStyle', 'LLVM')]))

    def test_yaml_2(self):
        res = whatstyle.parse_miniyaml("""\
---
IncludeCategories:
  - Regex:           '^"(llvm|llvm-c|clang|clang-c)/'
    Priority:        2
""")
        self.assertEqual(res,
                         OrderedDict([('IncludeCategories', [
                             OrderedDict([('Regex', '^"(llvm|llvm-c|clang|clang-c)/'),
                                          ('Priority', 2)])
                         ])]))

    def test_yaml_3(self):
        res = whatstyle.parse_miniyaml("""\
---
IncludeCategories:
  - Regex:           '^"(llvm|llvm-c|clang|clang-c)/'
    Priority:        2
IndentCaseLabels: false
""")
        self.assertEqual(res,
                         OrderedDict([('IncludeCategories', [
                             OrderedDict([('Regex', '^"(llvm|llvm-c|clang|clang-c)/'),
                                          ('Priority', 2)])
                         ]), ('IndentCaseLabels', False)]))

    def test_yaml_map_list(self):
        res = whatstyle.parse_miniyaml("""\
---
Language:        Cpp
# BasedOnStyle:  LLVM
AccessModifierOffset: -2
BraceWrapping:
  AfterClass:      false
  AfterControlStatement: false
IncludeCategories:
  - Regex:           '^"(llvm|llvm-c|clang|clang-c)/'
    Priority:        2
  - Regex:           '.*'
    Priority:        1
IndentCaseLabels: false
...
""")
        # yapf: disable
        expected = OrderedDict([('Language', 'Cpp'),
                                ('BasedOnStyle', 'LLVM'),
                                ('AccessModifierOffset', -2),
                                ('BraceWrapping', OrderedDict([
                                    ('AfterClass', False),
                                    ('AfterControlStatement', False)])),
                                ('IncludeCategories',
                                 [OrderedDict([(
                                     'Regex', '^"(llvm|llvm-c|clang|clang-c)/'),
                                     ('Priority', 2)]),
                                  OrderedDict([(
                                     'Regex', '.*'),
                                     ('Priority', 1)])]),
                                ('IndentCaseLabels', False)])
        # yapf: enable
        self.assertMultiLineEqual(self.jenc(res), self.jenc(expected))

    def test_yaml_inline_list(self):
        res = whatstyle.parse_miniyaml("""\
ForEachMacros:   [ foreach, Q_FOREACH, BOOST_FOREACH ]
""")
        # yapf: disable
        expected = OrderedDict([('ForEachMacros', ["foreach", "Q_FOREACH", "BOOST_FOREACH"])])
        # yapf: enable
        self.assertMultiLineEqual(self.jenc(res), self.jenc(expected))

    def test_yaml_plain_list(self):
        res = whatstyle.parse_miniyaml("""\
---
ForEachMacros:
  - foreach
  - Q_FOREACH
  - BOOST_FOREACH
...
""")
        # yapf: disable
        expected = OrderedDict([('ForEachMacros', ["foreach", "Q_FOREACH", "BOOST_FOREACH"])])
        # yapf: enable
        self.assertMultiLineEqual(self.jenc(res), self.jenc(expected))


# ---------------------------------------------------------------

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

try:
    from setuptools import setup, Command
except ImportError:
    from distutils.core import setup, Command


class RunTest(Command):

    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        import sys
        import subprocess
        errno = subprocess.call([sys.executable, 'tests/test_whatstyle.py'])
        raise SystemExit(errno)


with open('README.rst') as f:
    readme = f.read()

setup(
    name='whatstyle',
    version='0.1.9',
    url='https://github.com/mikr/whatstyle',
    license='MIT',
    author='Michael Krause',
    author_email='michael@krause-software.com',
    description='whatstyle finds a code format style that fits given source files.',
    long_description=readme,
    py_modules=['whatstyle'],
    cmdclass={'test': RunTest},
    zip_safe=False,
    platforms='any',
    keywords='formatter beautifier clang-format yapf tidy indent astyle uncrustify '
             'scalariform scalafmt rfmt rustfmt',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Software Development :: Quality Assurance',
        'Topic :: Utilities',
    ],
    entry_points={
        'console_scripts': [
            'whatstyle = whatstyle:main'
        ]
    },
)



#!/usr/bin/env python
# A tool to parse the FormatStyle struct from Format.h and update the
# documentation in ../ClangFormatStyleOptions.rst automatically.
# Run from the directory in which this file is located to update the docs.

import collections
import os
import re
import urllib2

CLANG_DIR = os.path.join(os.path.dirname(__file__), '../..')
FORMAT_STYLE_FILE = os.path.join(CLANG_DIR, 'include/clang/Format/Format.h')
DOC_FILE = os.path.join(CLANG_DIR, 'docs/ClangFormatStyleOptions.rst')


def substitute(text, tag, contents):
  replacement = '\n.. START_%s\n\n%s\n\n.. END_%s\n' % (tag, contents, tag)
  pattern = r'\n\.\. START_%s\n.*\n\.\. END_%s\n' % (tag, tag)
  return re.sub(pattern, '%s', text, flags=re.S) % replacement

def doxygen2rst(text):
  text = re.sub(r'<tt>\s*(.*?)\s*<\/tt>', r'``\1``', text)
  text = re.sub(r'\\c ([^ ,;\.]+)', r'``\1``', text)
  text = re.sub(r'\\\w+ ', '', text)
  return text

def indent(text, columns, indent_first_line=True):
  indent = ' ' * columns
  s = re.sub(r'\n([^\n])', '\n' + indent + '\\1', text, flags=re.S)
  if not indent_first_line or s.startswith('\n'):
    return s
  return indent + s

class Option:
  def __init__(self, name, type, comment):
    self.name = name
    self.type = type
    self.comment = comment.strip()
    self.enum = None
    self.nested_struct = None

  def __str__(self):
    s = '**%s** (``%s``)\n%s' % (self.name, self.type,
                                 doxygen2rst(indent(self.comment, 2)))
    if self.enum:
      s += indent('\n\nPossible values:\n\n%s\n' % self.enum, 2)
    if self.nested_struct:
      s += indent('\n\nNested configuration flags:\n\n%s\n' %self.nested_struct,
                  2)
    return s

class NestedStruct:
  def __init__(self, name, comment):
    self.name = name
    self.comment = comment.strip()
    self.values = []

  def __str__(self):
    return '\n'.join(map(str, self.values))

class NestedField:
  def __init__(self, name, comment):
    self.name = name
    self.comment = comment.strip()

  def __str__(self):
    return '\n* ``%s`` %s' % (
        self.name,
        doxygen2rst(indent(self.comment, 2, indent_first_line=False)))

class Enum:
  def __init__(self, name, comment):
    self.name = name
    self.comment = comment.strip()
    self.values = []

  def __str__(self):
    return '\n'.join(map(str, self.values))

class EnumValue:
  def __init__(self, name, comment):
    self.name = name
    self.comment = comment

  def __str__(self):
    return '* ``%s`` (in configuration: ``%s``)\n%s' % (
        self.name,
        re.sub('.*_', '', self.name),
        doxygen2rst(indent(self.comment, 2)))

def clean_comment_line(line):
  match = re.match(r'^/// \\code(\{.(\w+)\})?$', line)
  if match:
    lang = match.groups()[1]
    if not lang:
      lang = 'c++'
    return '\n.. code-block:: %s\n\n' % lang
  if line == '/// \\endcode':
    return ''
  return line[4:] + '\n'

def read_options(header):
  class State:
    BeforeStruct, Finished, InStruct, InNestedStruct, InNestedFieldComent, \
    InFieldComment, InEnum, InEnumMemberComment = range(8)
  state = State.BeforeStruct

  options = []
  enums = {}
  nested_structs = {}
  comment = ''
  enum = None
  nested_struct = None

  for line in header:
    line = line.strip()
    if state == State.BeforeStruct:
      if line == 'struct FormatStyle {':
        state = State.InStruct
    elif state == State.InStruct:
      if line.startswith('///'):
        state = State.InFieldComment
        comment = clean_comment_line(line)
      elif line == '};':
        state = State.Finished
        break
    elif state == State.InFieldComment:
      if line.startswith('///'):
        comment += clean_comment_line(line)
      elif line.startswith('enum'):
        state = State.InEnum
        name = re.sub(r'enum\s+(\w+)\s*\{', '\\1', line)
        enum = Enum(name, comment)
      elif line.startswith('struct'):
        state = State.InNestedStruct
        name = re.sub(r'struct\s+(\w+)\s*\{', '\\1', line)
        nested_struct = NestedStruct(name, comment)
      elif line.endswith(';'):
        state = State.InStruct
        field_type, field_name = re.match(r'([<>:\w(,\s)]+)\s+(\w+);',
                                          line).groups()
        option = Option(str(field_name), str(field_type), comment)
        options.append(option)
      else:
        raise Exception('Invalid format, expected comment, field or enum')
    elif state == State.InNestedStruct:
      if line.startswith('///'):
        state = State.InNestedFieldComent
        comment = clean_comment_line(line)
      elif line == '};':
        state = State.InStruct
        nested_structs[nested_struct.name] = nested_struct
    elif state == State.InNestedFieldComent:
      if line.startswith('///'):
        comment += clean_comment_line(line)
      else:
        state = State.InNestedStruct
        nested_struct.values.append(NestedField(line.replace(';', ''), comment))
    elif state == State.InEnum:
      if line.startswith('///'):
        state = State.InEnumMemberComment
        comment = clean_comment_line(line)
      elif line == '};':
        state = State.InStruct
        enums[enum.name] = enum
      else:
        raise Exception('Invalid format, expected enum field comment or };')
    elif state == State.InEnumMemberComment:
      if line.startswith('///'):
        comment += clean_comment_line(line)
      else:
        state = State.InEnum
        enum.values.append(EnumValue(line.replace(',', ''), comment))
  if state != State.Finished:
    raise Exception('Not finished by the end of file')

  for option in options:
    if not option.type in ['bool', 'unsigned', 'int', 'std::string',
                           'std::vector<std::string>',
                           'std::vector<IncludeCategory>']:
      if enums.has_key(option.type):
        option.enum = enums[option.type]
      elif nested_structs.has_key(option.type):
        option.nested_struct = nested_structs[option.type]
      else:
        raise Exception('Unknown type: %s' % option.type)
  return options

options = read_options(open(FORMAT_STYLE_FILE))

options = sorted(options, key=lambda x: x.name)
options_text = '\n\n'.join(map(str, options))

contents = open(DOC_FILE).read()

contents = substitute(contents, 'FORMAT_STYLE_OPTIONS', options_text)

with open(DOC_FILE, 'wb') as output:
  output.write(contents)
